//■■TextIO.java

import java.io.*;
import java.util.LinkedList;
import java.util.List;
import java.nio.file.Files;
import java.nio.file.Path;
import java.awt.*;
import javax.swing.*;

public class TextIO
{

    public static File callFileChooser(String mode, Component parent, String dir, String ext)
    {
        JFileChooser fileChooser = new JFileChooser(dir);
        if(!ext.equals("*"))
        {
          SingleFileFilter filter = new SingleFileFilter(ext);
          fileChooser.setFileFilter(filter);
        }
        int ret;
        if(mode.equals("open"))
            ret = fileChooser.showOpenDialog(parent);
        else if(mode.equals("save"))
            ret = fileChooser.showSaveDialog(parent);
        else
            return null;
        File file = fileChooser.getSelectedFile();
        if(ret != JFileChooser.APPROVE_OPTION || file == null)
            return null;
        else
            return file;
    }

    public static String[] loadStrArr(File file)
    { //デフォルト文字コード
        BufferedReader reader = null;
        Object[] objArr = null;
        String[] strArr = null;
        List<String> list = new LinkedList<String>();
        try
        {
            reader = new BufferedReader(new FileReader(file));
            try
            {
                String line  = "";
                for(int i=0;;i++) //データ読取り
                {
                    line = reader.readLine();
                    if(line == null) //終端超越なら
                        break;
                    list.add(line);
                }
                objArr = list.toArray();
                strArr = new String[objArr.length];
                for(int i=0; i<objArr.length; i++)
                {
                    strArr[i] = (String)objArr[i];
                }
            }
            catch(IOException e)
            {
                System.err.println(file.getName() + ": read error");
                strArr = null;
            }
            reader.close();
        }
        catch(FileNotFoundException e)
        {
            System.err.println(file.getName() + ": open error");
            strArr = null;
        }
        catch(IOException e)
        {
            System.err.println(file.getName() + ": close error");
            strArr = null;
        }
        return strArr;
    }

     public static String loadStr(File file)
    { //デフォルト文字コード
        BufferedReader reader = null;
        String retStr = "";
        try
        {
            reader = new BufferedReader(new FileReader(file));
            try
            {
                String line  = "";
                for(int i=0;;i++) //データ読取り
                {
                    line = reader.readLine();
                    if(line == null) //終端超越なら
                        break;
                    retStr = retStr + line + "\n";
                }
            }
            catch(IOException e)
            {
                System.err.println(file.getName() + ": read error");
                retStr = null;
            }
            reader.close();
        }
        catch(FileNotFoundException e)
        {
            System.err.println(file.getName() + ": open error");
            retStr = null;
        }
        catch(IOException e)
        {
            System.err.println(file.getName() + ": close error");
            retStr = null;
        }
        return retStr;
    }

    public static boolean saveStr(File file, String data)
    { //デフォルト文字コード
        BufferedWriter writer = null;
        boolean b = true;
        try
        {
            writer = new BufferedWriter(new FileWriter(file));
            try
            {
                writer.write(data);
                writer.flush();
            }
            catch(IOException e)
            {
                System.err.println(file.getName() + ": write error");
                b = false;
            }
            writer.close();
        }
        catch(IOException e)
        {
            if(writer == null)
            {
                System.err.println(file.getName() + ": open error");
            }
            else
            {
                System.err.println(file.getName() + ": close error");
            }
            b = false;
        }
        return b;
    }

    public static void saveText(File outputFile, String data, String charCode)
    { //charCode: "UTF-16", "UTF-8", "Shift_JIS", "MS932"
        PrintWriter pw = null; 
        try //●書き出しファイルの準備
        {
            pw = new PrintWriter(new OutputStreamWriter(
                new FileOutputStream(outputFile), charCode));
        }
        catch(Exception e)
        {
            System.err.println("Exception1: saveText() " + 
                charCode + "> \n" + e.toString());
        } 
        
        try //●ファイルへの書き出し 
        {
            if(data==null)
                System.err.println("data==null");
            if(pw==null)
                System.err.println("pw==null");
            if(data!=null && pw!=null)
                pw.print(data);
            pw.close();
        }
        catch(Exception e)
        {
            System.err.println("Exception2: saveText() " + 
                charCode + "> \n" + e.toString());
        } 
    }

    public static String loadText(File inputFile, String charCode)
    { //charCode: "UTF-16", "UTF-8", "Shift_JIS", "MS932"
        try //●ファイルからの読み取り
        {
            BufferedReader br = new BufferedReader(
                new InputStreamReader(new FileInputStream(inputFile), 
                charCode));
            String text1 = "";
            String text2 = "";
            while ((text1 = br.readLine()) != null) 
            {
                text2 = text2 + text1 + "\n";
            }
            return text2;
        }
        catch(Exception e)
        {
            System.err.println("Exception2: loadText() " + 
                charCode + "> \n" + e.toString());
            return null;
        } 
    }
}
